package com.unitesk.requality.core;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

public abstract class TreeStorage {
    protected IStorageDeltaListener deltaTarget;

    public TreeStorage() {
    }
    public void setDeltaTarget(IStorageDeltaListener deltaProcessor) {
        this.deltaTarget = deltaProcessor;
    }

    protected IStorageDeltaListener getDeltaTarget() {
        return deltaTarget;
    }

    public void createNode(UUID parentNode, UUID node, String type, String id) { 
        if(parentNode==null) {
            type = "DBRoot";
        }
        Set<Attribute> attr = new HashSet<Attribute>();
        attr.add(new Attribute(AttributeType.STRING, "_type",type));
        createNode(parentNode, node, id, attr);
    }

    public String getNodeType(UUID uuid) {
        for(Attribute a : getAttributes(uuid)) {
            if(a.getKey().equals("_type")) {
                return (String) a.getValue();
            }
        }
        return null;
    }

    abstract public UUID getRootNode();
    abstract public UUID getParent(UUID uuid);
    abstract public Collection<UUID> getChildren(UUID uuid);
    abstract public void createNode(UUID parentUUId, UUID uuid, String id, Collection<Attribute> attrs);
    abstract public void remove(UUID node);
    abstract public String getNodeId(UUID uuid);
    abstract public void changeNodeId(UUID node, String id);
    abstract public Collection<Attribute> getAttributes(UUID uuid);
    abstract public void saveAttributes(UUID node, Collection<Attribute> attributes);
    abstract public void moveNodeTo(UUID node, UUID parent);
    abstract public boolean canLoad(UUID uuid);
    abstract public String getNodeResourcePath(UUID uuId);
}
