/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.dialogs;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import com.ibm.icu.text.CharsetDetector;
import com.ibm.icu.text.CharsetMatch;
import com.unitesk.requality.eclipse.views.actions.documents.ImportAction;
import com.unitesk.requality.nodetypes.DocFolder;

public class ImportDialog extends Dialog {
    private static String TITLE = "Import";
    private static Image ICON = PlatformUI.getWorkbench().getSharedImages()
            .getImage(ISharedImages.IMG_OBJ_ELEMENT);

    private static final String LBL_FILE = "Import file:";
    private static final String LBL_SAVE = "Save to:";
    private static final String LBL_ENCODING = "Encoding:";

    private static final String BTN_FILE = "Browse...";
//    private static final String BTN_SAVE = "Browse...";

    private static String MES_NOTEXIST = "File does not exist.";

    private static String[] exts = { "*.htm;*.html;*.xhtml","*.*" };
    private static String[] extNames = { "Markup files", "All files" };

    private Combo fileText = null;
    private Text saveText = null;
    private Combo encCombo = null;

    private String[] filenames = null;
    private Map<String,String> encoding = new HashMap<String, String>();
    private DocFolder docFolder;

//    private final SelectionListener saveList = new SelectionListener() {
//        public void widgetDefaultSelected(SelectionEvent e) {
//        }
//
//        public void widgetSelected(SelectionEvent e) {
//            // FIXME: 
////            File docFolderFile = new File(docFolder.getTreeDB().getDocPath());
////            File docFolderFile = new File("/");
////            SelectDirDialog dirSelect = new SelectDirDialog(getShell(),
////                    docFolderFile, saveText.getText());
////            if (dirSelect.open() != Dialog.OK)
////                return;
////            File dir = dirSelect.getResult();
////            if (dir != null) {
////                docFolder = new DocFolder(docFolder.getTreeDB(), dir);
////                saveText.setText(docFolder.getRelativePath());
////            }
//        }
//    };

    private final SelectionListener butList = new SelectionListener() {
        public void widgetDefaultSelected(SelectionEvent e) {
        }

        public void widgetSelected(SelectionEvent e) {
            FileDialog fileSelect = new FileDialog(getShell(), SWT.MULTI);
            if(ImportAction.initalPath!=null) {
                fileSelect.setFilterPath(ImportAction.initalPath);
            }
            fileSelect.setFileName(fileText.getText());
            fileSelect.setFilterExtensions(exts);
            fileSelect.setFilterNames(extNames);
            if(fileSelect.open()!=null) {
        	for(String fileName : fileSelect.getFileNames()) {
        	    fileName = fileSelect.getFilterPath()+"/".concat(fileName);
        	    fileText.add(fileName);
        	    String enc = detectEncoding(fileName);
        	    if(enc != null) {
        		encoding.put(fileName, enc);
        		selectEncoding(enc);
        	    }
        	}
        	fileText.select(0);
            }
        }
    };

    private final SelectionListener encodingSelect = new SelectionListener() {
	public void widgetDefaultSelected(SelectionEvent e) {
        }
	
	public void widgetSelected(SelectionEvent e) {
	    encoding.put(fileText.getText(), encCombo.getText());
	}
    };
    
    private final SelectionListener fileSelect = new SelectionListener() {
	public void widgetDefaultSelected(SelectionEvent e) {
        }
	
	public void widgetSelected(SelectionEvent e) {
	    String enc = encoding.get(fileText.getText());
	    selectEncoding(enc);
	}
    };
    
    private String detectEncoding(String pathToFile) {
        FileInputStream in;
        try {
            in = new FileInputStream(pathToFile);
            CharsetDetector detector = new CharsetDetector();
            detector.enableInputFilter(true);
            detector.setText(new BufferedInputStream(in));
            CharsetMatch match = detector.detect();
            in.close();
            return match.getName();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }
    
    public ImportDialog(DocFolder docDir) {
        super(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell());
        docFolder = docDir;
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Composite container = new Composite(parent, SWT.NONE);
        container.setLayout(new GridLayout(2, false));

        Label fileLabel = new Label(container, SWT.NONE);
        fileLabel.setText(LBL_FILE);

        Composite fileContainer = new Composite(container, SWT.NONE);
        fileContainer.setLayout(new GridLayout(2, false));
        fileText = new Combo(fileContainer, SWT.BORDER);
        fileText.setLayoutData(new GridData(200, -1));
        fileText.addSelectionListener(fileSelect);
        Button fileBut = new Button(fileContainer, SWT.NONE);
        fileBut.setText(BTN_FILE);
        fileBut.addSelectionListener(butList);

        Label saveLabel = new Label(container, SWT.NONE);
        saveLabel.setText(LBL_SAVE);

        Composite saveContainer = new Composite(container, SWT.NONE);
        saveContainer.setLayout(new GridLayout(2, false));
        saveText = new Text(saveContainer, SWT.BORDER);
        saveText.setLayoutData(new GridData(200, -1));
        saveText.setEditable(false);
        saveText.setText(docFolder.getQualifiedId());

        Label encLabel = new Label(container, SWT.NONE);
        encLabel.setText(LBL_ENCODING);
        Composite encContainer = new Composite(container, SWT.NONE);
        encContainer.setLayout(new GridLayout(2, false));
        encCombo = new Combo(encContainer, SWT.READ_ONLY);
        encCombo.setLayoutData(new GridData(200, -1));
        for(String csName : Charset.availableCharsets().keySet()) {
            encCombo.add(csName);
        }
        encCombo.addSelectionListener(encodingSelect);
        
        selectEncoding("UTF-8");

        this.getShell().setText(TITLE);
        this.getShell().setImage(ICON);
        return container;
    }

    public int selectEncoding(String name) {
        String[] charsetNames = (String[]) Charset.availableCharsets().keySet().toArray(new String[0]);
        for(int i = 0; i < charsetNames.length; i++) {
            if(encCombo.getItem(i).equals(name)) {
                encCombo.select(i);
                return i;
            }
        }
        return -1;
    }
    
    @Override
    protected void okPressed() {
        filenames = fileText.getItems().clone();
        for(String filename : filenames) {
            File file = new File(filename);
            if (file.exists() && file.isFile()) {
        	super.okPressed();
            } else {
        	MessageDialog.openInformation(this.getShell(), TITLE, MES_NOTEXIST);
            }
        }
    }

    public String[] getFiles() {
        if(filenames.length!=0)
            return filenames;
        else
            return new String[0];
    }

    public String getEncoding(String filename) {
        String enc = encoding.get(filename);
        if(enc != null)
            return enc;
        else
            return "UTF-8";
    }
}
