/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.views.listeners;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.part.ViewPart;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

public class TreeViewerDropTargetListener implements DropTargetListener {

    private static final String CHANGE_PARENT_REQUIREMENT_MSG = "Are you sure you want to change a parent requirement ";
    private TreeViewer tree;
    private ViewPart view;

    @SuppressWarnings("unused")
    private TreeViewerDropTargetListener() {
    }

    public TreeViewerDropTargetListener(TreeViewer tree, ViewPart view) {
        this.tree = tree;
        this.view = view;
    }

    public void dragEnter(DropTargetEvent event) {
        event.detail = DND.DROP_MOVE;
    }

    public void dragLeave(DropTargetEvent event) {
    }

    public void dragOperationChanged(DropTargetEvent event) {
    }

    public void dragOver(DropTargetEvent event) {
        event.feedback = DND.FEEDBACK_SELECT | DND.FEEDBACK_EXPAND
                | DND.FEEDBACK_SCROLL;
        event.detail = canDropTo(event) ? DND.DROP_MOVE : DND.DROP_NONE;

    }

    private boolean canDropTo(DropTargetEvent event) {
        if (event.item == null)
            return false;
        if (!(event.item instanceof TreeItem))
            return false;

        TreeItem item = (TreeItem) event.item;
        if ((item.getData() instanceof TreeDB))
            item.setData(((TreeDB) item.getData()).getNode(Requirement
                    .getTypeRootQId()));
        if (!(item.getData() instanceof TreeNode))
            return false;

        TreeSelection selection = (TreeSelection) tree.getSelection();
        Requirement src = (Requirement) selection.getFirstElement();

        if (!(src instanceof TreeNode))
            return false;
        TreeNode dst = (TreeNode) item.getData();
        if (src.getTreeDB() != dst.getTreeDB())
            return false;

        // we can't deep node
        for (TreeNode node : new DeepFirstTreeWalker(src)) {
            if (node.getQualifiedId().equals(dst.getQualifiedId()))
                return false;
        }

        return true;
    }

    public void drop(DropTargetEvent event) {
        if (!canDropTo(event))
            return;
        TreeItem parItem = (TreeItem) event.item;
        Requirement newParent = (Requirement) parItem.getData();
        TreeSelection selection = (TreeSelection) tree.getSelection();
        Requirement src = (Requirement) selection.getFirstElement();

        if (newParent.getTreeDB() != src.getTreeDB())
            return;
        if (newParent.findChild(src.getId()) != null) {
            MessageDialog.openError(view.getSite().getShell(), "Requality",
                    "Child with same ID already exists!");
            return;
        }

        if (MessageDialog.openQuestion(view.getSite().getShell(), "Requality",
                CHANGE_PARENT_REQUIREMENT_MSG + " of '" + src.getName()
                        + "' to '" + newParent.getName() + "'?") == true) {

            src.setParent(newParent);
        }
    }

    public void dropAccept(DropTargetEvent event) {
    }

}
