package com.unitesk.requality.eclipse.views.properties;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.nodetypes.Requirement;

public class MainPropertySection extends RequirementPropertiesTab {

    private AttributesPanel attribPane;
    private Text textId;
    private Text textName;

    public MainPropertySection() {
    }

    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        TabbedPropertySheetWidgetFactory wf = getWidgetFactory();

        parent.setLayout(new FormLayout());
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));
        FormAttachment topSpace1 = new FormAttachment(0, 8);
        FormAttachment topSpace2 = new FormAttachment(0, 32);
        FormAttachment leftSpace1 = new FormAttachment(0, 8);
        FormAttachment leftSpace2 = new FormAttachment(0, 50);
        FormAttachment leftSpace3 = new FormAttachment(0, 60);
        FormAttachment rightSpace1 = new FormAttachment(100, -8);

        FormData labelIdData = new FormData();
        labelIdData.top=topSpace1;
        labelIdData.right=leftSpace2;
        FormData labelNameData = new FormData();
        labelNameData.top = topSpace2;
        labelNameData.right=leftSpace2;
        FormData labelAttribData = new FormData();
        labelAttribData.top=new FormAttachment(0,48);
        labelAttribData.left=leftSpace1;
        
        FormData textIdData = new FormData(0, SWT.DEFAULT);
        textIdData.top = topSpace1;
        textIdData.left = leftSpace3;
        textIdData.right = rightSpace1;

        FormData textNameData = new FormData(0, SWT.DEFAULT);
        textNameData.top = topSpace2;
        textNameData.left = leftSpace3;
        textNameData.right = rightSpace1;

        FormData panelData = new FormData();
        panelData.left = leftSpace1;
        panelData.right = rightSpace1;
        panelData.top = new FormAttachment(0, 64);
        panelData.bottom = new FormAttachment(100,-36);

        wf.createLabel(parent, "Id:").setLayoutData(labelIdData);
        textId = wf.createText(parent, "hello");
        textId.setLayoutData(textIdData);
        textId.addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
                if (req.getParent() instanceof Requirement)
                {
                    req.setId(textId.getText());
                    req.saveAttributes();
                }
            }
        });
        wf.createLabel(parent, "Name:").setLayoutData(labelNameData);
        textName = wf.createText(parent, "node name");
        textName.setLayoutData(textNameData);
        textName.addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
                if (!textName.getText().equals(textId.getText())) {
                    req.setName(textName.getText());
                } else {
                    req.removeAttribute("_name");
                }
                req.saveAttributes();
            }
        });

        attribPane = new AttributesPanel(parent, wf);
        attribPane.getControl().setLayoutData(panelData);
    }

    @Override
    public void refresh() {
        super.refresh();
        if (req != null) {
            textId.setText(req.getId());
            textName.setText(req.getName());
        }
    }

    @Override
    protected void setRequirement(Requirement req) {
        if (req.getParent() instanceof Requirement)
            textId.setEnabled(true);
        else
            textId.setEnabled(false);
        attribPane.setRequirement(req);
    }
}
