package com.unitesk.requality.eclipse.views.properties;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.properties.tabbed.AbstractPropertySection;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.Requirement;

public class SourcePropertySection extends AbstractPropertySection {
    private TextViewer source;
    private FontRegistry fontRegistry;
    public SourcePropertySection() {
        fontRegistry = new FontRegistry(PlatformUI.getWorkbench().getDisplay());
        fontRegistry.put("source", new FontData[]{new FontData("Monospaced", 12, SWT.NORMAL)});
    }

    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        parent.setLayout(new GridLayout());
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));
        source = new TextViewer(parent, SWT.MULTI | SWT.READ_ONLY);
        source.getControl().setFont(fontRegistry.get("source"));
        source.getControl().setLayoutData(new GridData(GridData.FILL_BOTH));
    }

    @Override
    public void setInput(IWorkbenchPart part, ISelection selection) {
        super.setInput(part, selection);
        if(selection.isEmpty()) {
            return;
        }
        if (!(selection instanceof IStructuredSelection)) {
            return;
        }
        IStructuredSelection ss = (IStructuredSelection) selection;
        if (!(ss.getFirstElement() instanceof Requirement)) {
            return;
        }
        Requirement req = (Requirement) ss.getFirstElement();
        IFile f = (IFile) TreesTracker.getResourceByNode(req);
        Writer writer = new StringWriter();
        char[] buffer = new char[1024];
        try {
            InputStream in = f.getContents();
            Reader reader = new BufferedReader(new InputStreamReader(in, f.getCharset()));
            int n;
            while ((n = reader.read(buffer)) != -1) {
                writer.write(buffer, 0, n);
            }
            source.setInput(new Document(writer.toString()));
            in.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
