package com.unitesk.requality.merge;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Document;

public class Tree {

    private List<Node> levelOrderedTraversal;

    protected Tree() {
	levelOrderedTraversal = new ArrayList<Node>();
    }

    public Tree(org.w3c.dom.Node domNode) {
	makeLevelOrderedTraversalSequence(domNode);
    }

    public Tree(Document doc) {
	makeLevelOrderedTraversalSequence(doc.getDocumentElement());
    }

    protected void makeLevelOrderedTraversalSequence(org.w3c.dom.Node root) {
	levelOrderedTraversal = new ArrayList<Node>();
	levelOrderedTraversal.add(new Node(root));
	for (int i = 0; i < levelOrderedTraversal.size(); i++) {
	    Node currentNode = levelOrderedTraversal.get(i);
	    if (!currentNode.getDomNode().hasChildNodes())
		continue;
	    for (org.w3c.dom.Node x = currentNode.getDomNode().getFirstChild(); x
		    .getNextSibling() != null; x = x.getNextSibling())
		levelOrderedTraversal.add(new Node(currentNode, x));
	    levelOrderedTraversal.add(new Node(currentNode, currentNode
		    .getDomNode().getLastChild()));
	}
    }

    public Iterator<Node> traverse() {
	return levelOrderedTraversal.listIterator();
    }

    public List<Node> getTraversalSequence() {
	return levelOrderedTraversal;
    }

    public Node getRoot() {
	return levelOrderedTraversal.get(0);
    }

    public interface NodeFilter {
	public boolean accept(Node x, Node referenceNode);
    }

    public List<Node> nearestMatch(Node x) {
	return nearestMatch(x, this.levelOrderedTraversal);
    }

    public List<Node> nearestMatch(Node x, NodeFilter f) {
	List<Node> filteredList = new ArrayList<Node>();
	for (Node node : this.levelOrderedTraversal)
	    if (f.accept(node, x))
		filteredList.add(node);
	return nearestMatch(x, filteredList);
    }

    public List<Node> nearestMatch(Node x, Node parent) {
	return nearestMatch(x, parent.getDescendants());
    }

    public List<Node> nearestMatch(Node x, Node parent, NodeFilter f) {
	List<Node> filteredList = new ArrayList<Node>();
	for (Node node : parent.getDescendants())
	    if (f.accept(node, x))
		filteredList.add(node);
	return nearestMatch(x, filteredList);
    }

    private List<Node> nearestMatch(Node x, List<Node> possibleMatches) {
	if (possibleMatches.isEmpty())
	    return new ArrayList<Node>();
	final Context refContext = x.getContext();
	final Comparator<Node> POSITIONAL_ORDER = new Comparator<Node>() {
	    public int compare(Node x, Node y) {
		return x.getContext().positionDistance(refContext)
			- y.getContext().positionDistance(refContext);
	    }
	};
	final Comparator<Node> SIMILARITY_ORDER = Collections
		.reverseOrder(new Comparator<Node>() {
		    public int compare(Node x, Node y) {
			return x.getContext().contextSimilarity(refContext)
				- y.getContext().contextSimilarity(refContext);
		    }
		});
	Collections.sort(possibleMatches, POSITIONAL_ORDER);
	Collections.sort(possibleMatches, SIMILARITY_ORDER);
	if (possibleMatches.size() > 3)
	    return possibleMatches.subList(0, 2);
	return possibleMatches;
    }

    public boolean isEqual(Tree t) {
	if (this.levelOrderedTraversal.size() != t.levelOrderedTraversal.size())
	    return false;
	Iterator<Node> i = this.traverse();
	Iterator<Node> j = t.traverse();
	while (i.hasNext() && j.hasNext()) {
	    Node reference = i.next();
	    Node match = j.next();
	    if (!reference.isEqual(match))
		return false;
	}
	return true;
    }

    public void updateTree() {
	makeLevelOrderedTraversalSequence(getRoot().getDomNode());
    }
}
