package com.unitesk.requality.tests.core;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import org.junit.Before;
import org.junit.Test;

import com.unitesk.requality.core.MemoryStorage;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.TreeStorage;

public class APITest {
    private TreeDB db;

    @Before
    public void setUp() {
        db = new TreeDB(createStorage());
    }
    
    protected TreeStorage createStorage() {
        return new MemoryStorage();
    }
    
    // TODO: protect root from id change (id's storage task)
    // boolean exception = false;
    // try{
    //     root.setId("not_root");
    // } catch (Exception e) {
    //     exception = true;
    // }
    // assertTrue(exception);
    
    // Create root
    @Test
    public void rootCreation() {
        TreeNode root = getDB().getRootNode();
        assertEquals("Root node must same",
                root, getDB().getRootNode());
        assertEquals("Root node type must be 'DBRoot'", 
                "DBRoot", root.getType());
        assertEquals("Root node type must be 'DBRoot'",
                getDB().getNodeType("DBRoot").getName(), root.getType());
        assertNull("Root parent is always null", root.getParent());
        assertEquals("root id is always '/'",
                "/", root.getId());


    }

    protected TreeDB getDB() {
        return db;
    }

    // Create node
    // TODO: (tests) node type value must be name one of /Types node child
    @Test
    public void nodeCreation() {
        TreeNode root = getDB().getRootNode();

        assertEquals("node must not have children", 0, root.sizeChildren());

        TreeNode a = root.createChild("a");
        assertNotNull(a);
        assertEquals("node default type is parent type or 'Node' if parent is root",
                "Node", a.getType());
        assertFalse("node type must not be 'DBRoot'", 
                getDB().getNodeType("DBRoot").equals(a.getType()));
        assertEquals("node must not have children", 0, a.sizeChildren());

        TreeNode aa = a.createChild("aa");

        assertEquals("node default type is parent type or 'Node' if parent is root",
                "Node", aa.getType());
        assertFalse("node type must not be 'DBRoot'", 
                getDB().getNodeType("DBRoot").equals(aa.getType()));
        assertEquals("node must not have children", 0, aa.sizeChildren());

        TreeNode ab = a.createChild("ab");

        assertEquals("node default type is parent type or 'Node' if parent is root",
                "Node", ab.getType());
        assertEquals("node must not have children", 0, ab.sizeChildren());

        TreeNode b = root.createChild("b", "NodeType");
        
        assertEquals("node default type is parent type or 'Node' if parent is root",
                "NodeType", b.getType());
        assertEquals("node must not have children", 0, b.sizeChildren());

        TreeNode ba = b.createChild("ba");
        
        assertEquals("node default type is parent type or 'Node' if parent is root",
                "NodeType", ba.getType());
        
        assertNotNull("node must have parent", a.getParent());
        assertNotNull("node must have parent", b.getParent());
        assertNotNull("node must have parent", aa.getParent());
        assertNotNull("node must have parent", ab.getParent());
        assertEquals("node must have parent", root, a.getParent());
        assertEquals("node must have parent", root, b.getParent());
        assertEquals("node must have parent", a, aa.getParent());
        assertEquals("node must have parent", a, ab.getParent());

        boolean thrown = false;
        try {
            a.createChild("aa");
        } catch (Exception e) {
            thrown = true;
        }
        assertTrue("node id must be different from node's brothers", 
                thrown);
        assertEquals("node id must be different from node's brothers",
                2, a.sizeChildren());
    }
    @Test(expected=IllegalArgumentException.class)
    public void nodeWithSameIdCreation() {
        TreeNode root = getDB().getRootNode();
        root.createChild("hello");
        root.createChild("hello");
    }

    // 3. Create node attribute
    // 3.1 if node attribute key is already binded with other value, old value repla
    // 4. Create node with respect to node type
    // ...

    // Read
    @Test
    public void readRootNode() {
        TreeNode root = getDB().getRootNode();
        TreeNode root1 = getDB().getRootNode();
        assertEquals(root, root1);
    }

    // 2. Read children
    @Test
    public void readChildrenOrdered() {
        TreeNode root = getDB().getRootNode();
        String childrenIds[] = {"ch3", "ch1", "ch2"};
        String childrenIdsSorted[] = {"ch1", "ch2", "ch3"};
        
        for(String id: childrenIds) {
            root.createChild(id);
        }

        TreeNode[] children = root.getSortedChildren();
        for(int i=0; i < children.length; i++) {
            assertEquals(childrenIdsSorted[i], children[i].getId());
        }
    }

    @Test
    public void readChildrenFiltered() {
        TreeNode root = getDB().getRootNode();
        String childrenIds[] = {"ch3", "ch1", "ch2"};
        String childrenTypes[] = {"Node", "Node", "NodeType"};
        String childrenIdsSorted[] = {"ch1", "ch3"};
        
        for(int i = 0; i < childrenIds.length; i++) {
            root.createChild(childrenIds[i], childrenTypes[i]);
        }

        TreeNode[] children = root.getSortedChildren("Node");
        assertEquals(2, children.length);
        for(int i=0; i < children.length; i++) {
            assertEquals(childrenIdsSorted[i], children[i].getId());
        }
    }
    // 2.2. 
    // ...
    // 3. Attribute read (type of attribute)
    // ...

    // 4. TreeWalker
//    @Test
//    public void deepFirstTreeWalker() {
//        TreeNode root = getDB().getRootNode();
//        TreeNode a = root.createChild("a");
//        TreeNode b = root.createChild("b");
//        TreeNode ab = a.createChild("ab");
//        TreeNode ac = a.createChild("ac");
//        int i=0;
//        String[] order = new String[] {"/", "a", "ab", "ac", "b"};
//        for(TreeNode cur : new DeepFirstTreeWalker(root)) {
//            assertEquals(order[i], cur.getId());
//            i++;
//        }
//    }
    // ...
    
    // Update
    // 1. Root constant
    // ...
    // 2. Change node id
    // ...
    // 3. Change attribute value
    // ...
    // 4. Change node parent
    // ...
    // 5. Change node type
    // ...
    // 6. Change control by type
    // ...
    // 7. Change node parent


    // Delete
    // 1. Delete root
    // ...
    // 2. Delete child
    // ...
    // 3. Node self delete
    // ...
    // 4. Remove attribute
    // ...
}
