package com.unitesk.requality.tests.core;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.Set;

import org.junit.Test;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.MemoryStorage;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.TreeStorage;


public class AttributesTest {
    @Test
    public void attributeCRUD() {
        TreeStorage storage = new MemoryStorage();
        TreeDB db = new TreeDB(storage);
        TreeNode node = db.getRootNode();

        // Create
        // 1. Creat new attribute
        Attribute a = new Attribute(AttributeType.STRING, "a", "a");
        Attribute prev = node.putAttribute(a);
        assertNull(prev);
        assertEquals(a, node.getAttribute("a"));
        assertEquals(AttributeType.STRING, node.getAttribute("a").getType());
        // 2. Create attribute with same name
        Attribute b = new Attribute(AttributeType.STRING, "a", "b");
        prev = node.putAttribute(b);
        assertNotNull(prev);
        assertEquals(a, prev);
        assertEquals(b, node.getAttribute("a"));
        
        // Read
        // 1. Read existing
        String aVal = (String) node.getAttributeValue("a");
        assertNotNull(aVal);
        assertEquals("b", aVal);
        // 2. Read unexisting
        Object val = node.getAttribute("unexisting");
        assertNull(val);
        val = node.getAttributeValue("unexisting");
        assertNull(val);
        // 3. Read all
        node.putAttribute(new Attribute(AttributeType.STRING, "a", "a"));
        node.putAttribute(new Attribute(AttributeType.STRING, "b", "b"));
        Set<String> keys = node.getAttributeKeys();
        assertEquals(2+1, keys.size()); // _type
        assertTrue(keys.contains("a"));
        assertTrue(keys.contains("b"));

        // Update
        int attrCount = node.getAttributeKeys().size();
        Object oldValue = node.getAttributeValue("a");
        assertNotNull(oldValue);
        node.putAttribute(new Attribute(AttributeType.STRING, "a", null));
        assertNull(node.getAttributeValue("a"));
        node.putAttribute(new Attribute(AttributeType.STRING, "a", "aval"));
        assertEquals("aval", node.getAttributeValue("a"));
        assertEquals(attrCount, node.getAttributeKeys().size());

        // Delete
        attrCount = node.getAttributeKeys().size();
        Attribute attr = node.getAttribute("a"); 
        assertNotNull(attr);
        Attribute attrWas = node.removeAttribute(attr.getKey());
        assertEquals(attr, attrWas);
        assertEquals(attrCount-1, node.getAttributeKeys().size());
        assertNull(node.getAttributeValue(attr.getKey()));
    }

    // TODO: (tests) attribute type test
    // lists
    // resources
}
