package com.unitesk.requality.tests.core;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.UUID;

import org.junit.Before;
import org.junit.Test;

import com.unitesk.requality.core.FSStorage;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tests.utils.Ulils;


public class LocationTest {
    private TreeDB db;
    private String locId1;
    private String locId2;

    public String getXhtlpWithLocation(String locUUID1, String locUUID2) {
        StringBuffer sb = new StringBuffer();
        sb.append("<html xmlns=\"http://www.w3.org/1999/xhtml\"><head><title>Sample.html</title></head>");
        sb.append("<body>");
        sb.append("<span class=\"requality_text id_"+locUUID1+"\">");
        sb.append("<a name=\""+locUUID1+"\" id=\"id_"+locUUID1+"\" class=\"requality_id\"/>Hello, World!");
        sb.append("</span>");
        sb.append("<span class=\"requality_text id_"+locUUID2+"\">");
        sb.append("<a name=\""+locUUID2+"\" id=\"id_"+locUUID2+"\" class=\"requality_id\"/>Hello, World!");
        sb.append("</span>");
        sb.append("</body></html>");
        return sb.toString();
    }

    @Before
    public void setUp() throws IOException {
        String path = System.getProperty("java.io.tmpdir") + File.separator
                + "treedb.test";
        if (new File(path).exists()) {
            Ulils.recursiveDelete(new File(path));
        }
        db = new TreeDB(new FSStorage(path));
        db.addNodeType("Document", Document.class);
        db.addNodeType("DocFolder", DocFolder.class);
        db.addNodeType("Location", Location.class);
        db.addNodeType("Requirement", Requirement.class);
        TreeNode docRoot = db.getRootNode().createChild("Documents", "Document");
        db.getRootNode().createChild("Requirements", "Requirement");
        Document doc = (Document) docRoot.createChild("test.xhtml");
        System.out.println(doc.getQualifiedId());
        new File(doc.getHTMLPath()).getParentFile().mkdirs();
        new File(doc.getHTMLPath()).createNewFile();
        FileWriter fw = new FileWriter(doc.getHTMLPath());
        
        locId1=UUID.randomUUID().toString();
        locId2=UUID.randomUUID().toString();
 
        doc.createChild(locId1,"Location");
        doc.createChild(locId2,"Location");
        fw.write(getXhtlpWithLocation(locId1,locId2));
        fw.close();
    }

    // To string tests
    @Test
    public void stringMaker() {
//        Document doc = (Document) db.getNode("/Documents/test.xhtml");
        Location loc1 = (Location) db.getNode("/Documents/test.xhtml/"+locId1);
        Location loc2 = (Location) db.getNode("/Documents/test.xhtml/"+locId2);
        assertEquals("orphan location(/Documents/test.xhtml/"+locId2+")", loc2.toString());
        Requirement req1 = (Requirement) db.getNode(Requirement.getTypeRootQId()).createChild("1");
//        Requirement req2 = (Requirement) db.getNode(Requirement.getTypeRootQId()).createChild("2");
        req1.addLocation("/Documents/test.xhtml/"+locId1);
        req1.addLocation("/Documents/test.xhtml/"+locId2);
        System.out.println(loc1.toString());
        System.out.println(loc2.toString());
        // TODO
    }
}
