///*
// * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
// *
// * Licensed under the Apache License, Version 2.0 (the "License");
// * you may not use this file except in compliance with the License.
// * You may obtain a copy of the License at
// *
// *     http://www.apache.org/licenses/LICENSE-2.0
// *
// * Unless required by applicable law or agreed to in writing, software
// * distributed under the License is distributed on an "AS IS" BASIS,
// * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// * See the License for the specific language governing permissions and
// * limitations under the License.
// */
//
//package com.unitesk.requality.tests.depricated;
//
//import static org.junit.Assert.*;
//
//import java.io.File;
//import java.util.Iterator;
//import java.util.UUID;
//
//import org.junit.After;
//import org.junit.Before;
//import org.junit.Test;
//
//import com.unitesk.requality.core.TreeDB;
//import com.unitesk.requality.core.TreeDBIO;
//import com.unitesk.requality.core.TreeDBNode;
//import com.unitesk.requality.nodetypes.Requirement;
//import com.unitesk.requality.tools.DeepFirstTreeWalker;
//import com.unitesk.requality.tools.TreeWalker;
//
//public class TreeDBTest {
//
//    private String path = System.getProperty("java.io.tmpdir") + File.separator
//            + "treedb.test";
//    private TreeDB db;
//
//    @Before
//    public void setUp() {
//        if (new File(path).exists())
//            recursiveDelete(new File(path));
//        db = new TreeDBIO(path);
//    }
//
//    @After
//    public void tearDown() {
//        recursiveDelete(new File(path));
//    }
//
//    @Test
//    public void createNodeRoot() {
//        TreeDBNode root = db.createRootNode(Requirement.type);
//        assertEquals(0, root.sizeChildren());
//        assertEquals(
//                "Type root node must have same user id as it's type folder",
//                Requirement.type.getName(), root.getId());
//        assertNotNull(root);
//        assertEquals("Second requiest for type root must return same id", root,
//                db.getRootNode(Requirement.type, false));
//
//        db.save(root);
//        assertEquals(0, db.getRootNode(Requirement.type, false).sizeChildren());
//        File rootFile = new File(db.getRootPath(Requirement.type)
//                + File.separator + Requirement.type.getFileName());
//
//        assertTrue("Root requiremnt file exists", rootFile.exists());
//        assertTrue("Only root file must exists ", rootFile.getParentFile()
//                .listFiles().length == 1);
//
//        db.clear();
//        UUID restoredRootId = db.getRootNode(Requirement.type, false).getUUId();
//        assertEquals(root.getUUId(), restoredRootId);
//    }
//
//    @Test
//    public void createSubNode() {
//        TreeDBNode root = db.getRootNode(Requirement.type, false);
//        TreeDBNode req = db.createNode(root, "req");
//        assertNotNull(req);
//        assertFalse(root.getUUId().equals(req.getUUId()));
//        req = (Requirement) db.getNode(req.getUUId());
//
//        assertNotNull(req);
//        assertEquals(req.getUUId(), req.getUUId());
//        assertEquals(root.getUUId(), req.getParent());
//
//        db.saveAll();
//
//        String reqPath = db.getRootPath(Requirement.type) + File.separator
//                + req.getId() + File.separator + Requirement.type.getFileName();
//
//        assertTrue(new File(reqPath).exists());
//        assertEquals("root node file and subnode folder must be here", 2,
//                new File(db.getRootPath(Requirement.type)).listFiles().length);
//    }
//
//    @Test
//    public void deleteNode() {
////        UUID newNode = db.createNode(db.getRootNode(Requirement.type, false)).getId();
////
////        assertEquals("Node was added to root as first child", 1, db.getNode(
////                db.getRoot(Requirement.type)).getChildren().size());
////        UUID target = db.getNode(db.getRoot(Requirement.type)).getChildren()
////                .get(0);
////        assertNotNull(target);
////        assertEquals(newNode, target);
////
////        db.saveAll();
////
////        boolean result = db.delete(target);
////        assertTrue(result);
////        assertEquals(0, db.getNode(db.getRoot(Requirement.type)).getChildren()
////                .size());
//    }
//
//    @Test
//    public void fileOperationTest() {
//        String predicate = "a==b";
//
//        Requirement node = (Requirement) db.createNode(db
//                .createRootNode(Requirement.type), "node");
//        TreeDBNode base = (TreeDBNode) db.createNode(db.getRootNode(
//                Requirement.type, false), "base");
//        node.setId("SimpleReq");
//        node.addAttribute("min_version", "1.0");
//        node.addAttribute("max_version", "2.0");
//        node.setDescription("This is a node description");
//        node.setPredicate(predicate);
//        node.addBaseNode(base.getUUId());
//
//        TreeDBNode child = (TreeDBNode) db.createNode(node, "child");
//        child.setId("Child");
//
//        db.saveAll();
//        db.clear();
//
//        TreeDBNode restoredNode = (TreeDBNode) db.getNode(node.getUUId());
//        assertNotNull("Restored node in null", restoredNode);
//        MapperTest.assertNodeEquals(node, restoredNode);
//    }
//
//    @Test
//    public void newNodeSave() {
//        TreeDBNode root = db.createRootNode(Requirement.type);
//        assertEquals(db.getRootNode(Requirement.type, false), root);
//        Requirement req = Requirement.create(root, "req");
//        assertNotNull(req);
//        db.save(req);
//        assertNotNull(db.getNode(req.getUUId()));
//    }
//
//    @Test
//    public void changeNodeName() {
//        TreeDBNode root = db.createRootNode(Requirement.type);
//        TreeDBNode middleReq = db.createNode(root, "middleReq");
//        middleReq.setId("Middle");
//        Requirement req = (Requirement) db.createNode(middleReq, "req");
//        req.setId("req1");
//        // add child for more fun
//        db.createNode(req, "nnn");
//        db.saveAll();
//
//        File oldFile = new File(db.getNodeFilePath(req));
//
//        assertTrue(oldFile.exists());
//
//        req.setId("req_new");
//        db.saveAll();
//
//        File newFile = new File(db.getNodeFilePath(req));
//
//        // reloadTreeDB();
//
//        req = (Requirement) db.getNode(req.getUUId());
//
//        assertEquals("req_new", req.getId());
//        assertFalse(oldFile.exists());
//        assertFalse(oldFile.getParentFile().exists());
//        assertTrue(newFile.exists());
//    }
//
//    public void deepFirstTreeWalk() {
//        Requirement root = (Requirement) db.createRootNode(Requirement.type);
//        Requirement a = Requirement.create(db.getRootNode(Requirement.type,
//                false), "a");
//        Requirement b = Requirement.create(db.getRootNode(Requirement.type,
//                false), "b");
//        Requirement aa = Requirement.create(a, "aa");
//        Requirement ab = Requirement.create(a, "ab");
//        Requirement ba = Requirement.create(b, "ba");
//        Requirement bb = Requirement.create(b, "bb");
//
//        TreeWalker tw = new DeepFirstTreeWalker(db.getRootNode(
//                Requirement.type, false));
//        Iterator<TreeDBNode> i = tw.iterator();
//
//        assertTrue(i.hasNext());
//        assertEquals(i.next(), root);
//        assertTrue(i.hasNext());
//        assertEquals(i.next(), a);
//        assertTrue(i.hasNext());
//        assertEquals(i.next(), aa);
//        assertTrue(i.hasNext());
//        assertEquals(i.next(), ab);
//        assertTrue(i.hasNext());
//        assertEquals(i.next(), b);
//        assertTrue(i.hasNext());
//        assertEquals(i.next(), ba);
//        assertTrue(i.hasNext());
//        assertEquals(i.next(), bb);
//        assertFalse(i.hasNext());
//    }
//
//}
